﻿//////////////////////////////////////////////
// Scheduler.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkTasks
{
	class Task ;
	class Thread ;
}

/// Includes ---------------------------------

// nkTasks
#include "../Dll/DllDefines.h"

// Standards
#include <mutex>
#include <vector>

/// Class ------------------------------------

namespace nkTasks
{
	class DLL_TASKS_EXPORT Scheduler final
	{
		private :
		
			// La liste des threads internes
			std::vector<Thread*> _threads ;
			// Compteur interne
			unsigned int _taskCounter ;

			// Mutex pour les tâches threads
			std::mutex _taskMutex ;
			// Tâches terminées par les threads
			std::vector<Task*> _resolvedTasks ;
			
		public :
		
			// Constructeur, destructeur
			Scheduler (size_t threadCount = 0) ;
			~Scheduler () ;

			// Getters
			size_t getThreadCount () const ;
			Thread* getThread (size_t index) const ;

			// Ajout d'une task
			void addTask (Task* task) ;

			// Fonction pour les threads : ajout de tâche terminée
			void resolveTask (Task* task) ;

			// Check des threads qui ont fini
			// A appeler dans le thread qui a besoin du boulot fait
			void checkEndedTasks () ;

			// Threads
			void joinAllThreads () ;
			void resizeForThreadCount (size_t threadCount) ;
	} ;
}